<?php

/**
 * @file
 * Configuration for PHPMailer, newsletter content, formats, and subscriptions.
 */

/**
 * Generates a form for PHPMailer settings.
 */
function sunmailer_admin_phpmailer() {
  drupal_add_js(drupal_get_path('module', 'sunmailer') . '/sunmailer.jquery.js');
  $form = array();

  // Important settings.
  $form['phpmailer_path'] = array(
    '#type' => 'textfield',
    '#title' => t('PHPMailer path'),
    '#description' => t('Path to the directory containing class.phpmailer.php - leave blank if PHPMailer is in PHP\'s include path'),
    '#default_value' => variable_get('sunmailer_phpmailer_path', ''),
  );
  $form['test_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Test email'),
    '#description' => t('Email address to send test emails to'),
    '#default_value' => variable_get('sunmailer_test_email', ''),
  );
  $form['send'] = array(
    '#type' => 'submit',
    '#value' => t('Send test email'),
  );

  // Basic settings.
  $form['basic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Basic settings'),
    '#collapsible' => FALSE,
  );
  $form['basic']['from_name'] = array(
    '#type' => 'textfield',
    '#title' => t('From name'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_from_name', ''),
  );
  $form['basic']['from_email'] = array(
    '#type' => 'textfield',
    '#title' => t('From email'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_from_email', ''),
  );
  $max_recipients = variable_get('sunmailer_max_recipients', 1);
 	$form['basic']['batch_emails'] = array(
    '#type' => 'radios',
    '#title' => t('Email batching'),
    '#description' => t('The mail server may need to be configured to support a high volume of emails, but batching emails via Bcc can trigger spam filters'),
    '#options' => array(
      FALSE => 'Send one email for each email address, putting the address in the To field',
      TRUE => 'Send emails to groups of email addresses, putting the addresses in the Bcc field',
    ),
    '#default_value' => (int)($max_recipients != 1),
  );
 	$form['basic']['max_recipients'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum batch size'),
    '#description' => t('Maximum number of email adresses to Bcc in a single email - use 0 for no limit'),
    '#required' => TRUE,
    '#default_value' => ($max_recipients == 1) ? 0 : $max_recipients,
 	);

  // Mail settings.
  $form['mail'] = array(
    '#type' => 'fieldset',
    '#title' => t('Mail settings'),
    '#collapsible' => FALSE,
  );
  $form['mail']['mail_method'] = array(
    '#type' => 'radios',
    '#title' => t('Mailing method'),
    '#description' => t('Selects the method used to mail the newsletter'),
    '#required' => TRUE,
    '#options' => array(
      'mail' => 'mail',
      'sendmail' => 'sendmail',
      'smtp' => 'SMTP',
    ),
    '#default_value' => variable_get('sunmailer_method', 'mail'),
  );

  // Mail settings - Sendmail configuration.
  $form['mail']['sendmail'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sendmail configuration'),
    '#collapsible' => FALSE,
    '#prefix' => '<div id="sendmail-fieldset">',
    '#suffix' => '</div>',
  );
  $form['mail']['sendmail']['sendmail_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Sendmail path'),
    '#description' => t('Path to the sendmail program'),
    '#default_value' => variable_get('sunmailer_sendmail_path', '/usr/bin/sendmail'),
  );

  // Mail settings - SMTP configuration.
  $form['mail']['smtp'] = array(
    '#type' => 'fieldset',
    '#title' => t('SMTP configuration'),
    '#collapsible' => FALSE,
    '#prefix' => '<div id="smtp-fieldset">',
    '#suffix' => '</div>',
  );
  $form['mail']['smtp']['host'] = array(
    '#type' => 'textfield',
    '#title' => t('Host'),
    '#description' => t('URL or IP address of the server sending mail'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_host', 'localhost'),
  );
  $form['mail']['smtp']['port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port'),
    '#description' => t('Port the server sends mail on'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_port', 25),
  );
  $form['mail']['smtp']['smtp_auth'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_auth', FALSE),
  );
  $form['mail']['smtp']['smtp_user'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('Username for SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_user', ''),
  );
  $form['mail']['smtp']['smtp_pass'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#description' => t('Password for SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_pass', ''),
  );
  $form['mail']['smtp']['smtp_secure'] = array(
    '#type' => 'radios',
    '#title' => t('Connection security'),
    '#options' => array(
      '' => t('None'),
      'ssl' => 'SSL',
      'tls' => 'TLS',
    ),
    '#default_value' => variable_get('sunmailer_smtp_secure', ''),
  );

  // Advanced settings.
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['advanced']['retries'] = array(
    '#type' => 'textfield',
    '#title' => t('Retry attempts'),
    '#description' => t('Number of times to retry if the message fails to be sent'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_retries', 2),
  );
  $form['advanced']['char_set'] = array(
    '#type' => 'textfield',
    '#title' => t('Character set'),
    '#description' => t('Character encoding to use for emails'),
    '#required' => TRUE,
    '#default_value' => 'utf-8',
  );
  $form['advanced']['word_wrap'] = array(
    '#type' => 'textfield',
    '#title' => t('Word wrap'),
    '#description' => t('For text emails, maximum number of text characters on one line'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_word_wrap', 80),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for PHPMailer settings form.
 */
function sunmailer_admin_phpmailer_validate($form, &$form_state) {
  $values = $form_state['values'];

  // Validate top-level settings.
  $found_phpmailer = FALSE;
  if ($path = $values['phpmailer_path']) {
    if (file_exists("$path/class.phpmailer.php")) {
      $found_phpmailer = TRUE;
    }
  }
  else {
    $paths = explode(':', get_include_path());
    foreach ($paths as $path) {
      if (file_exists("$path/class.phpmailer.php")) {
        $found_phpmailer = TRUE;
        break;
      }
    }
  }
  if (!$found_phpmailer) {
    form_set_error('phpmailer_path', t('Cannot find PHPMailer.'));
  }
  if ($form_state['clicked_button']['#value'] == t('Send test email')) {
    if (!$values['test_email']) {
      form_set_error('test_email', t('Test email is required to send a test email.'));
    }
  }

  // Validate basic settings.
  if (!is_numeric($values['max_recipients']) || $values['max_recipients'] < 0) {
    form_set_error('max_recipients', t('Maximum batch size must be a positive integer.'));
  }

  // Validate mail settings.
  if (!is_numeric($values['port']) || $values['port'] < 0) {
    form_set_error('port', t('Port must be a positive integer.'));
  }
  switch ($values['method']) {
  case 'sendmail':
    if (!$values['sendmail_path']) {
      form_set_error('sendmail_path', t('Sendmail path is required if using Sendmail.'));
    }
    break;
  case 'smtp':
    if ($values['smtp_auth']) {
      if (!$values['smtp_user']) {
        form_set_error('smtp_user', t('SMTP username is required for SMTP authentication.'));
      }
      if (!$values['smtp_pass']) {
        form_set_error('smtp_pass', t('SMTP password is required for SMTP authentication.'));
      }
    }
    break;
  }

  // Validate advanced settings.
  if (!is_numeric($values['retries']) || $values['retries'] < 0) {
    form_set_error('retries', t('Resend attempts must be a positive integer.'));
  }
  if (!is_numeric($values['word_wrap']) || $values['word_wrap'] < 0) {
    form_set_error('word_wrap', t('Word wrap must be a positive integer.'));
  }
}

/**
 * Submission callback for PHPMailer settings form.
 */
function sunmailer_admin_phpmailer_submit($form, &$form_state) {
  $values = $form_state['values'];

  // Set top-level settings.
  variable_set('sunmailer_phpmailer_path', $values['phpmailer_path']);
  variable_set('sunmailer_test_email', $values['test_email']);

  // Set basic settings.
  variable_set('sunmailer_from_name', $values['from_name']);
  variable_set('sunmailer_from_email', $values['from_email']);
  variable_set('sunmailer_host', $values['host']);
  variable_set('sunmailer_port', (int)$values['port']);
  if ($values['batch_emails']) {
    variable_set('sunmailer_max_recipients', (int)$values['max_recipients']);
 	}
 	else {
    variable_set('sunmailer_max_recipients', 1);
 	}

  // Set mail settings.
  variable_set('sunmailer_mail_method', $values['mail_method']);
  variable_set('sunmailer_sendmail_path', $values['sendmail_path']);
  variable_set('sunmailer_smtp_auth', (bool)$values['smtp_auth']);
  variable_set('sunmailer_smtp_user', $values['smtp_user']);
  variable_set('sunmailer_smtp_pass', $values['smtp_pass']);
  variable_set('sunmailer_smtp_secure', $values['smtp_secure']);

  // Set advanced settings.
  variable_set('sunmailer_retries', (int)$values['retries']);
  variable_set('sunmailer_char_set', $values['char_set']);
  variable_set('sunmailer_word_wrap', (int)$values['word_wrap']);

  // Send a test email if requested.
  if ($form_state['clicked_button']['#value'] == t('Send test email')) {
    $recipients = array(variable_get('sunmailer_test_email', ''));
    $mailer = new SunMailerMailer();
    $mailer->sendMail($recipients, 'SunMailer Test', print_r($mailer, TRUE), FALSE);
  }
}
