<?php
/**
 * @file
 * Contains the SunMailerMailer class.
 */

/**
 * Used to send mail.
 */
class SunMailerMailer {
  /**
   * PHPMailer instance backing the mailer.
   *
   * @var PHPMailer
   */
  private $mailer;

  /**
   * Initializes a new instance of the SunMailerMailer class.
   */
  public function __construct() {
    // Create a PHPMailer instance.
    $path = variable_get('sunmailer_phpmailer_path', '');
    require_once $path ? "$path/class.phpmailer.php" : 'class.phpmailer.php';
    $this->mailer = new PHPMailer();

    // Set the sender.
    $this->mailer->FromName = variable_get('sunmailer_from_name', '');
    $this->mailer->From = variable_get('sunmailer_from_email', '');

    // Configure how mail is sent.
    $this->mailer->Host = variable_get('sunmailer_host', 'localhost');
    $this->mailer->Port = variable_get('sunmailer_port', 25);
    switch (variable_get('sunmailer_method', 'mail')) {
    case 'mail':
      $this->mailer->IsMail();
      break;
    case 'sendmail':
      $this->mailer->IsSendmail();
      $this->mailer->Sendmail = variable_get('sunmailer_sendmail_path', '/usr/bin/sendmail');
      break;
    case 'smtp':
      $this->mailer->IsSMTP();
      $this->mailer->SMTPAuth = variable_get('sunmailer_smtp_auth', FALSE);
      if ($this->mailer->SMTPAuth) {
        $this->mailer->Username = variable_get('sunmailer_smtp_user', '');
        $this->mailer->Password = variable_get('sunmailer_smtp_pass', '');
      }
      $this->mailer->SMTPSecure = variable_get('sunmailer_smtp_secure', '');
      break;
    }

    // Set some advanced settings.
    $this->mailer->CharSet = variable_get('sunmailer_char_set', 'utf-8');
    $this->mailer->WordWrap = variable_get('sunmailer_word_wrap', 80);
  }

  /**
   * Sends mail.
   *
   * @param $recipients
   *   Recipient list.
   * @param $subject
   *   Subject line for the email.
   * @param $body
   *   Body of the email.
   * @param $isHtml
   *   Whether or not the body is HTML or text.
   * @param $altBody
   *   (Optional) Alternate body for HTML emails.
   */
  public function sendMail($recipients, $subject, $body, $isHtml, $altBody = NULL) {
    // Set the mail content.
    $this->mailer->Subject = $subject;
    $this->mailer->Body = $body;
    $this->mailer->IsHTML($isHtml);
    $this->mailer->AltBody = $alt_body;

    // Send mail to all recipients.
    $recipientGroups = SunMailerMailer::splitRecipients($recipients);
    foreach ($recipientGroups as $recipientGroup) {
      // Set the recipients.
      $this->mailer->ClearAllRecipients();
      if (count($recipientGroup) == 1) {
        $this->mailer->AddAddress(array_pop($recipientGroup));
      }
      else {
        foreach ($recipientGroup as $recipient) {
          $this->mailer->AddBCC($recipient);
        }
      }

      // Send the mail.
      $retries = variable_get('sunmailer_retries', 2);
      while (TRUE) {
        if ($this->mailer->Send()) {
          break;
        }
        if (--$retries < 0) {
          watchdog('mail', 'SunMailer failed to send email (@error)', array('@error' => $mailer->ErrorInfo), WATCHDOG_ERROR);
          break;
        }
        watchdog('mail', 'SunMailer failed to send email, retrying... (@error)', array('@error' => $mailer->ErrorInfo), WATCHDOG_WARNING);
      }
    }
  }

  /**
   * Splits the recipient list into batches.
   *
   * @param $recipients
   *   Recipient list.
   */
  private static function splitRecipients($recipients) {
    $groupSize = variable_get('sunmailer_max_recipients', 1);
    $groupSize = $groupSize ? $groupSize : count($recipients);
    $recipientGroups = array();
    for ($i = 0; $i < count($recipients); $i += $groupSize) {
      $recipientGroups[] = array_slice($recipients, $i, $groupSize);
    }
    return $recipientGroups;
  }
}
